/*
 * tdzdd for s-t path with length bounds
 * by KUT_KMHT (Shuto Kato, Satoshi Matsumoto, Takashi Harada, Shogo Takeuchi)
 * Copyright (c) 2023 KUT_KMHT
 * original: TdZdd/apps/ddpaths/ddpaths.cpp
 * 
 * TdZdd: a Top-down/Breadth-first Decision Diagram Manipulation Framework
 * by Hiroaki Iwashita <iwashita@erato.ist.hokudai.ac.jp>
 * Copyright (c) 2014 ERATO MINATO Project
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <cstdlib>
#include <iomanip>
#include <iostream>
#include <map>
#include <stdexcept>
#include <string>
#include <boost/multiprecision/cpp_int.hpp>

#include <tdzdd/DdStructure.hpp>
#include "PathZdd.hpp"
#include <tdzdd/spec/PathZddByStdMap.hpp>
#include <tdzdd/spec/DegreeConstraint.hpp>
#include <tdzdd/spec/FrontierBasedSearch.hpp>
#include <tdzdd/spec/SizeConstraint.hpp>
#include "Graph.hpp"

using namespace tdzdd;

std::map<std::string,bool> opt;
std::map<std::string,int> optNum;
std::map<std::string,std::string> optStr;

bool one_pair;
int nodes_num;
int limit;

void input(std::vector<std::vector<int>> &Edges, int &t1, int &t2) {
  std::string dump; // dump site
  int edges_num;
  std::cin >> dump >> dump >> nodes_num >> edges_num; // p edge xxx xxx
  std::vector<std::vector<int>> v_edge(edges_num, std::vector<int>(2));
  for (int i = 0; i < edges_num; i++) {
    std::cin >> dump >> v_edge[i][0] >> v_edge[i][1];
  }
  std::cin >> dump >> limit;
  std::cin >> dump;
  if (dump == "t") {
    one_pair = true;
    std::cin >> t1 >> t2;
  } else {
    one_pair = false;
  }

  Edges.resize(nodes_num+1);
  for (int i = 0; i < edges_num; i++) {
    Edges[v_edge[i][0]].push_back(v_edge[i][1]);
    Edges[v_edge[i][1]].push_back(v_edge[i][0]);
  }
}


int main(int argc, char *argv[]) {
  std::vector<std::vector<int>> Edges;
  int t1, t2;
  input(Edges, t1, t2);
  optNum["l"] = limit;

  if (one_pair) {

    try {
      modified::Graph graph;
      graph.readAdjacencyList(Edges);
      graph.readVertexGroups(t1, t2);

      int const n = graph.edgeSize();
      DdStructure<2> f(n);

      DdStructure<2> g = f;
      modified::PathZdd p(graph, true, optNum["l"]);
      f = DdStructure<2>(p, true);

      f.zddReduce();

      std::cout << f.evaluate(ZddCardinality<>()) << std::endl;

    }
    catch (std::exception& e) {
      std::cerr << e.what() << std::endl;
      return 1;
    }
  } else {
    boost::multiprecision::cpp_int ans = 0;
#ifdef _OPENMP
    int break_loop = 0;
#endif

    for (int outside = 1; outside < nodes_num; outside++) {
#ifdef _OPENMP
      if (break_loop > 0) break;
#pragma omp parallel for
#endif
      for (int inside = outside+1; inside < nodes_num+1; inside++) {
#ifdef _OPENMP
	if (break_loop > 0) continue;
#endif
	try {
	  modified::Graph graph;
	  graph.readAdjacencyList(Edges);
	  graph.readVertexGroups(outside, inside);

	  int const n = graph.edgeSize();
	  DdStructure<2> f(n);

	  DdStructure<2> g = f;
	  modified::PathZdd p(graph, true, optNum["l"]);
	  f = DdStructure<2>(p, false);

	  f.zddReduce();

	  // std::cout << f.evaluate(ZddCardinality<>()) << std::endl;
	  boost::multiprecision::cpp_int val(f.evaluate(ZddCardinality<>()));
#ifdef _OPENMP
#pragma omp critical
#endif
	  {
	    ans += val;
	  }

	}
	catch (std::exception& e) {
	  std::cerr << e.what() << std::endl;
#ifdef _OPENMP
	  break_loop++;
	  continue;
#else
	  return 1;
#endif
	}
	
      }
    }
#ifdef _OPENMP
    if (break_loop > 0) return 1;
#endif
    std::cout << ans << std::endl;
  }
  
  return 0;
}
